"use strict";

// By default citeproc-js formats some output with div classes "csl-entry", "csl-left-margin", etc.
// Instead we change the citeproc-js HTML format to the one that Mendeley plugins use.
// The original citeproc-js output is defined in citeproc.js src/formats.js
CSL.Output.Formats["html"]["@bibliography/entry"] = function (state, str) {
	return str;
};
CSL.Output.Formats["html"]["@display/right-inline"] = function (state, str) {
	return str;
};
CSL.Output.Formats["html"]["@display/left-margin"] = function (state, str) {
	return "<second-field-align>" + str + "</second-field-align>";
};

CSL.Output.Formats["rtf"]["@display/left-margin"] = function (state, str) {
	// display-left-margin is formatted by Mendeley Desktop
	return "\\display-left-margin" + str + "\\tab ";
};

var Mendeley_citeprocReact;

// Wraps functions exposed by Mendeley Desktop, providing explicit option names
// for better readability
var MendeleyDesktopWrapper = {
	setCitation: function (options) {
		MendeleyDesktop.setCitation(
			options.position,
			options.citation,
			options.citationId
		);
	},
	setBibliography: function (options) {
		MendeleyDesktop.setBibliography(
			options.bibliography,
			options.hangingIndent,
			options.hasBibliography
		);
	}
};

// Process a list of citation clusters and generate a list of formatted citations
// and a bibliography
var processCitations = function (style, locale, citationClusters, items, enumerateCitations, reset, bibliographyOutputFormat) {
	try {
		if (!Mendeley_citeprocReact) {
			Mendeley_citeprocReact = new CiteprocReact({
				citeprocSys: {
					stringCompare: MendeleyDesktop.localeAwareCaseInsensitiveStringCompare,
					getAbbreviation: function (dummy, obj, jurisdiction, vartype, key) {
						if (vartype == "container-title") {
							obj["default"][vartype][key] = MendeleyDesktop.getAbbreviation(key);
						}
					},
					retrieveLocale: function(lang) {
						return MendeleyDesktop.getLocaleXml(lang);
					}
				}
			});
		}

		// Render citations and bibliography
		var renderOptions = {
			style: style,
			locale: locale,
			citationClusters: citationClusters,
			items: items,
			enumerateCitations: enumerateCitations,
			reset: reset,
			bibliographyOutputFormat: bibliographyOutputFormat 
		};
		var result = Mendeley_citeprocReact.render(renderOptions);

		if (citationClusters.length === 0 && !result.formattedBibliography) {
			// Can't use "bibliographyOnly" mode for styles which don't have a bibliography element,
			// so as a workaround we need to add an in-line citation and use this as the
			// bibliography
			renderOptions.citationClusters = [{
					citationItems: [{
						id: _.keys(items)[0]
					}]
				}];
			result = Mendeley_citeprocReact.render(renderOptions);
		}

		// Pass result back to Mendeley Desktop
		_.each(result.formattedCitations, function (formattedCitation, index) {
			MendeleyDesktopWrapper.setCitation({
				position: index,
				citation: formattedCitation,
				citationId: "citationID not relevant"
			});
		});

		if (result.formattedBibliography) {
			MendeleyDesktopWrapper.setBibliography({
				bibliography: result.formattedBibliography,
				hangingIndent: (result.formattedBibliographyOptions && result.formattedBibliographyOptions.hangingindent)
					? true : false,
				hasBibliography: true
			});
		} else if (citationClusters.length === 0) {
			// This is a hack to allow Mendeley Desktop's "View library as citations" mode
			// to work with styles which don't include a bibliography
			MendeleyDesktopWrapper.setBibliography({
				bibliography: [result.formattedCitations[0]],
				hangingIndent: false,
				hasBibliography: true
			});
		} else {
			MendeleyDesktopWrapper.setBibliography({
				bibliography: "This citation style does not include a bibliography.",
				hangingIndent: false,
				hasBibliography: true
			});
		}
	}
	catch (err) {
		CSL.error("Exception: " + err);
	}
};

var citeprocVersion = function () {
	return CSL.PROCESSOR_VERSION;
};

